import time

from datetime import datetime
from calendar import monthrange
from netforce.model import Model, fields, get_model
from netforce.database import get_connection
from netforce.access import get_active_company

from . import utils

class ReportHDCaseSummary(Model):
    _name="clinic.report.hd.case.summary"
    _string="Hemodialysis Report Summary"
    _transient=True
    
    _fields={
        "date": fields.Date("Month", required=True),
    }

    _defaults={
        'date': lambda *a: time.strftime("%Y-%m-%d"),        
    }
    
    def get_report_data(self,ids,context={}):
        db=get_connection()
        company_id=get_active_company()
        company=get_model("company").browse(company_id)

        date=datetime.now().strftime("%Y-%m-%d")
        if ids:
            obj=self.browse(ids)[0]
            date=obj.date 
        year=int(date[0:4])
        prev_year=year
        next_year=year
        crr_month=int(date[5:7])
        prev_month=crr_month-1
        next_month=crr_month+1
        if crr_month==1:
            prev_month=12
            prev_year-=1
        if crr_month==12:
            next_month=1
            next_year+=1

        month_str=utils.MONTHS['th_TH'][crr_month]
        next_month_str=utils.MONTHS['th_TH'][next_month]
        prev_month_str=utils.MONTHS['th_TH'][prev_month]
              
        # number of hd case of this month
        dom=[]
        weekday, crr_total_day=monthrange(year, crr_month)
        time_start='%s-%s-01 00:00:00'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s 23:59:59'%(year,str(crr_month).zfill(2),crr_total_day)
        dom.append(['time_start','>=', time_start])
        dom.append(['time_stop','<=',time_stop])
        dom.append(['state','=','completed'])
        crr_total=len(get_model("clinic.hd.case").search(dom))
        items={}
        items['topic1']={
                'month': month_str,
                'amount': crr_total,
                'date_from': time_start[0:10],
                'date_to': time_stop[0:10],
                'link': True,
            }

        # number  patient from previous month
        dom=[]
        weekday, prev_total_day=monthrange(prev_year, prev_month)
        #time_start='%s-%s-01'%(prev_year,str(prev_month).zfill(2))
        time_stop='%s-%s-%s'%(prev_year,str(prev_month).zfill(2),prev_total_day)
        #dom.append(['reg_date','>=', time_start])
        dom.append(['reg_date','<=',time_stop])
        npatient=len(get_model("clinic.patient").search(dom))
        items['topic2']={
                'month': prev_month_str,
                'amount': npatient or 0.0,
                'date_from': '',
                'date_to': '',
                'link': False,
            }

        # new patient of this month
        dom=[]
        weekday, crr_total_day=monthrange(prev_year, crr_month)
        time_start='%s-%s-01 00:00:00'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s 23:59:59'%(year,str(crr_month).zfill(2),crr_total_day)
        new_patients=db.query("select id, name, note from clinic_patient where active=true and create_time >=%s and create_time <=%s and company_id=%s",time_start,time_stop,company_id)
        items['topic3']={
                'month': month_str,
                'amount': len(new_patients) or 0.0,
                'date_from': '',
                'date_to': '',
                'link': False,
            }

        # number for patient who resign for this month
        time_start='%s-%s-01 00:00:00'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s 23:59:59'%(year,str(crr_month).zfill(2),crr_total_day)
        resign_patients=db.query("select id, name, note from clinic_patient where active=false and write_time >=%s and write_time <=%s and company_id=%s",time_start,time_stop,company_id)
        items['topic4']={
                'month': month_str,
                'amount': len(resign_patients) or 0.0,
                'date_from': '',
                'date_to': '',
                'link': False,
            }
        # all patient who are in hospital now
        dom=[]
        weekday, crr_total_day=monthrange(year, crr_month)
        time_stop='%s-%s-%s'%(year,str(crr_month).zfill(2),crr_total_day)
        npatients=db.query("select id, name, note from clinic_patient where active=true and reg_date <=%s and company_id=%s",time_stop,company_id)
        items['topic5']={
                'month': next_month_str,
                'amount': len(npatients),
                'date_from': '',
                'date_to': '',
                'link': False,
            }

        npatients=get_model("clinic.patient").search([['type','=','sc']])
        items['topic6']={
                'month': '',
                'amount': len(npatients),
                'date_from': '',
                'date_to': '',
                'link': False,
            }
        npatients=get_model("clinic.patient").search([['type','=','nhso']])
        items['topic7']={
                'month':'',
                'amount': len(npatients),
                'date_from': '',
                'date_to': '',
                'link': False,
            }
        npatients=get_model("clinic.patient").search([['type','in',['personal','others']]])
        items['topic8']={
                'month': '',
                'amount': len(npatients),
                'date_from': '',
                'date_to': '',
                'link': False,
        }

        lines=[] 
        index=1
        for item in items:
            topic='topic%s'%index
            name=utils.TOPICS[topic]['name']
            unit=utils.TOPICS[topic]['unit']
            line=items[topic]
            line['topic']=name
            line['unit']=unit
            lines.append(line)
            index+=1

        new_patient_lines=[]
        index=1
        for new_patient in new_patients:
            new_patient_lines.append({
                'no': index,
                'name': new_patient['name'] or "",
                'note': new_patient['note'] or "",
            })
            index+=1

        resign_patient_lines=[]
        index=1
        for resign_patient in resign_patients:
            resign_patient_lines.append({
                'no': index,
                'name': resign_patient['name'],
                'note': resign_patient['note'],
            })
            index+=1

        data={
            'month': month_str,
            'year': year,
            'lines': lines,
            'new_patient_lines': new_patient_lines,
            'resign_patient_lines': resign_patient_lines,
            'company_name': company.name or "",
            'parent_company_name': company.parent_id.name or "",
        }
        return data

ReportHDCaseSummary.register()
